#ifndef LeoArg_hh
#define LeoArg_hh

#include <list>
#include <string>

/// a class for command line parsing
/** 
    \example leoargmain.cpp 
    There is an example of this class
*/
class LeoArg 
{
 public:
  typedef std::list<std::string>ArgList;  ///< arg list type
  typedef std::list<std::string>::iterator ArgListIt; ///< arg list type iterator
  
  struct Option ///< A structure representing one option
  {
    std::string long_option;  ///< the long option
    std::string short_option; ///< the short version of the option
    ArgList     arglist;      ///< the list of parameter
    bool        bool_value;   ///< the boolean value of a parameter
    int         int_num;      ///< the integer value of a parameter
    double      double_num;   ///< the double value of a parameter
    std::string str;          ///< the parameter as a string
    bool        flag;         ///< true if the parameter is set
    
    Option( std::string long_option, std::string short_option );
  };
  
  typedef std::list<Option>OptionList; ///< Option list type
  typedef OptionList::iterator OptionListIt; ///< Option list iterator
  
  struct InvalidOption ///< thrown if an invalid option was detected
  {
    Option option;     ///< the invalid option
    std::string err;   ///< an error string
    InvalidOption( Option &o, std::string e ) : option( o ), err( e ){}
  };
  
  struct InvalidSetting ///< thrown if an invalid setting was detected
  {
    std::string err;   ///< an error string
    InvalidSetting( std::string e ) : err( e ){}
  };
  
  struct Setting ///< structure handling a whole parameter setting
  {
    int argc; 
    char **argv;
    std::string long_option_switch;  ///< the long option switch, eg.: "--"
    std::string short_option_switch; ///< the short option switch, eg.: "-"
    mutable OptionList option_list;          ///< the list of options
    bool sos;                        ///< true of short option string enabled

    Setting( int argc, char **argv, std::string long_option_switch, std::string short_option_switch )
      : argc( argc ), argv( argv ), long_option_switch( long_option_switch ), short_option_switch( short_option_switch )
    { sos = false; }
    
    Setting()
      : argc(0), argv(0)
    { sos = false; }
    
    /// adds an new option to the option list
    void add( const Option& option ) { option_list.push_back( option ); }

    /// enable/disable the short option string
    /*** if you want to set varios short options and its easyer to set one short option string.
	 eg.: 
	 the options: -s -p -q  
	 the sort option string can be: -spq 
	 
	 \Warning only one character short options are possible
	 \Warning the short option string doesn't looks like the short option of an option
	 \Warning the option can only be a boolean option
    */
    void short_option_string( bool short_os ) { sos = short_os; }
    bool short_option_string() const { return sos; } ///< returns true if short option string parsing is enabled
  };
  
 private:
  Setting setting; ///< the setting
  
 public:
  LeoArg( int argc, char **argv, std::string long_option_switch = "--", std::string short_option_switch = "-" );
  LeoArg( Setting& setting ); ///< initialize with a filled setting structure
  
  /// enable/disable the short option string
  void short_option_string( bool short_os ) { setting.short_option_string(short_os); }  

  /// returns true if short option string parsing is enabled
  bool short_option_string() const { return setting.short_option_string(); } 
  
  Setting get_setting() const { return setting; } ///< returns the current setting
  
  bool has_parameters() const { return setting.argc > 1; } ///< true if any optin or parameter is set
  bool get_option( Option& option );                    ///< returns a specific option
  
  /// returns the first parameter
  /** eg.: "./cmd hello --file a_file another_file"
      the function will return "hello"
  */
  bool get_first( std::string& ret_value ) const;
  
  /// returns the first parameters
  /** eg.: "./cmd hello world --file a_file another_file"
      the function will fill the ArgList with "hello" and "world"
  */
  bool get_first( ArgList& arglist ) const;
  
  /// returns the last parameter
  /** eg.: "./cmd --file a_file another_file"
      the function will return "another_file"
  */
  bool get_last( std::string & ret_value ) const;
  
  /// returns the last parameters
  /** eg.: "./cmd --file a_file another_file"
      the function will fill the ArgList with "a_file" and "another_file"
  */
  bool get_last( ArgList& arglist ) const;
  
  std::string get_prog_name() const { return std::string(setting.argv[0]); }

  /// clear the cache
  void clear_cache() { setting.option_list.clear(); }
 private:
  bool parse_option( Option& option ) const;
  void correct_option( Option& option ) const;
  void correct_setting();
  bool find_option( Option& option ) const; ///< function searches in cache for the specific option
  bool is_in_sos( const Option& option ) const; ///< checks if an option is in the short option string
  int has_sos() const; ///< finds a short option string
};


extern "C" {
  /// library identify check function, used bei autoconf/automake
  /*** For autoconf check add this line to your configure.in file:
       AC_CHECK_LIB(LeoArg, leoarg, echo -n;, echo not found ;exit, -lstdc++)
  */
  char leoarg();

  /*** For autoconf check add this line to your configure.in file:
       AC_CHECK_LIB(LeoArg, leoarg_version_2_2, echo "LeoArg >= 2.2 (yes)", \
       echo "LeoArg >= 2.2 (no)"; exit, -lstdc++ )
  */
  char leoarg_version_2_2(); //< stil valid, cause class is backwards compatible
  char leoarg_version_2_2_1();
  // no need for definig a 2_2_2 version, cause interface and behavior didn't changed

}

#endif
